@@ -22,7 +22,7 @@
  * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA  02111-1307  USA
  *
- * $Id: scim_m17n_imengine.cpp,v 1.8 2004/08/24 14:39:48 suzhe Exp $
+ * $Id: scim_m17n_imengine.cpp,v 1.13 2005/04/04 10:17:00 suzhe Exp $
  */
 
 #define Uses_SCIM_UTILITY
@@ -51,7 +51,7 @@
     #define SCIM_M17N_ICON_FILE           (SCIM_ICONDIR "/scim-m17n.png")
 #endif
 
-static const size_t  __m17n_nr_uuids = 64;
+static const size_t  __m17n_nr_uuids = 128;
 static const char   *__m17n_uuids [] = {
 "de94c8e7-6021-4787-a7ed-0d505bf9eea3",
 "31cfe859-8015-4067-9b75-1615bd54c72d",
@@ -116,7 +116,71 @@ static const char   *__m17n_uuids [] = {
 "f74c872c-6ebe-4702-8ef7-c5fd271d36cf",
 "06155ca5-554b-45a7-a04a-907ea38c2979",
 "0a6c1b0c-0c9e-400b-81d0-69b0323fa655",
-"c79aed67-6d75-40af-b2d0-93a1cec5c596"
+"c79aed67-6d75-40af-b2d0-93a1cec5c596",
+"8f0c5c5b-9121-4962-9475-96767eae586e",
+"766656eb-6575-4e16-9d92-c382c1f5f5a2",
+"ff41d2ed-cb47-49b0-920e-06abd0ee277c",
+"7b49235d-7550-4cef-b1d1-0a62fad4bd2c",
+"34cecab5-1f22-431f-a895-e855cbd41bf0",
+"2adfb2f1-6242-4730-996a-18447081670c",
+"3d08446a-389c-4c98-aa84-13960d5cfd3e",
+"92fef7d9-14e1-44df-b5c5-8bd3e4d364af",
+"99bf476f-7746-4b9c-9940-d6f4ca1fcef5",
+"fc738e82-98fc-47a6-87d0-9de582e9f625",
+"8f4b8524-93f6-4792-ac03-f9ca5f0e49ef",
+"c3848b56-0e3c-454c-b859-dddb52a22c42",
+"f5117076-707e-41d5-aef3-381027b020bb",
+"e1dddc6b-c268-4dd5-8629-1d1bdff8efe2",
+"329e9e57-780f-4b1e-8c96-e206cae70fc6",
+"3ec7a83a-883c-4c6d-9f34-f616d7f66227",
+"9b0989dc-047d-4186-93d7-59c4a67792d3",
+"c4366de0-64bd-4672-8613-bf89c269e125",
+"5f11f45c-6545-4d3b-ad95-dc02a1827e55",
+"52ed1916-f753-4eca-89a4-3dd527f6e7dd",
+"5963628b-6cb6-4de5-a531-b05529c3b02f",
+"ed49a819-086f-4e48-b60a-815862049f87",
+"60a4641f-e585-4940-8142-c853c7c81e5a",
+"f85db69b-98b3-42bc-a8f7-c9aff51c93a4",
+"a8866a89-1a9d-4584-a562-3731bb678dae",
+"ab48f3fb-ec6e-4d16-bb89-845122b75f05",
+"5f3881f2-7313-48fa-8ed5-1f3714232829",
+"29cbbd59-ac94-461b-8fdd-537c28c50847",
+"68ddbf8d-ea8a-4163-96ba-e3f711385507",
+"513c0ae8-9a17-4c29-8241-75d803055058",
+"0a87a377-f361-40c7-b183-b74cf378d642",
+"9a418ce6-0ee6-459d-9f2e-69c1d33a854c",
+"d5f5fbcc-2496-41f7-aa5f-d934e88c72fc",
+"e439ccdc-d2a2-4949-b658-691be51866fc",
+"b5e808c0-72f7-4ba7-a4d7-29d8ee6f33b7",
+"022b8e62-5721-4144-8c90-0f87a194c3cb",
+"8db6baad-72cb-485f-859d-7a85d3e2ecc7",
+"1929ea7b-bdf3-4e63-8c81-ebe95e1274a1",
+"f25a4891-1a69-4190-8143-ba2e9d6dfa4e",
+"92a12eba-988c-47d4-a473-8a83814a142e",
+"38c502a8-8de1-47de-a9c1-f1f9a35f4689",
+"740218f0-8aca-433a-88c2-3b2e077df8e4",
+"98d3e84e-e78e-40e4-bf4c-bc95bfcb37a5",
+"b0bd43a3-c537-4934-802b-a4cccd20b7e7",
+"61d4b4ef-b1a1-42fd-9e00-da0e148501f0",
+"b29c4400-b880-4a03-9a80-a1d48dcb8b30",
+"4d00c0b3-4f1e-4b9e-832d-c5904d48e5d9",
+"a1e45eb5-fc35-4455-ab8a-71acbc33d95f",
+"e9c7013a-7630-4310-a4a1-c2ec67f76c7a",
+"04ab7e89-bdba-4f5e-9442-6f653acf04f2",
+"dee60281-0dbb-4d1b-a989-6ee89f4c9250",
+"c0ee3079-2c0c-4b4b-894f-beb5404162f5",
+"fb17e4fc-9901-4c44-9e7f-7b179276066d",
+"d502438f-4e0e-4d39-ab9b-883cf15695c2",
+"f3537cf5-6e82-4162-9eca-ffd1a9bd5bb9",
+"aa91350b-a0b7-4ce2-ac82-1c208dddcfce",
+"2d2d2a6f-64a6-49b4-af11-25c49857bbc6",
+"8ac094fb-2b1b-4eeb-be91-4c87f1c7e34b",
+"7ab49ede-33da-4ce2-9d6c-7ee83a61cc43",
+"1cab3d11-1fe8-4bbe-8759-4844939f486c",
+"f36a0c58-f71e-4539-a43e-cad371c71d52",
+"f6b7b374-be88-4022-ac10-d5c47998a9b4",
+"b6fd6dd7-f4fe-4860-b92d-272f8cba616c",
+"d1c01bd6-ad12-42b8-9b73-47e51fcff098"
 };
 
 struct M17NInfo {
@@ -350,6 +414,8 @@ M17NInstance::process_key_event (const KeyEvent& key)
 {
     if (!m_ic) return false;
 
+    if (key.is_key_release ()) return true;
+
     MSymbol m17n_key = __key_to_symbol (key);
 
     if (m17n_key == Mnil) return false;
@@ -445,6 +511,27 @@ void
 M17NInstance::focus_out ()
 {
     SCIM_DEBUG_IMENGINE(2) << "focus_out.\n";
+    minput_reset_ic(m_ic);
+
+    char buf [1024];
+    MText *produced;
+    int ret;
+    produced = mtext();
+    ret = minput_lookup(m_ic, Mnil, NULL, produced);
+
+    if (ret) {
+        SCIM_DEBUG_IMENGINE(3) << "minput_lookup returns 1\n";
+    }
+
+    mconv_rebind_buffer(__m17n_converter, (unsigned char *) buf, 1024);
+    mconv_encode(__m17n_converter, produced);
+    buf[__m17n_converter->nbytes] = 0;
+    m17n_object_unref(produced);
+
+    if (strlen (buf)) {
+        SCIM_DEBUG_IMENGINE(2) << "commit_string: " << buf << "\n";
+        commit_string (utf8_mbstowcs (buf));
+    }
 }
 
 void
@@ -518,9 +605,9 @@ M17NInstance::preedit_draw_cb  (MInputContext *ic, MSymbol command)
                                             SCIM_ATTR_DECORATE,
                                             SCIM_ATTR_DECORATE_REVERSE));
 
+            this_ptr->show_preedit_string ();
             this_ptr->update_preedit_string (wstr, attrs);
             this_ptr->update_preedit_caret (ic->cursor_pos);
-            this_ptr->show_preedit_string ();
         } else {
             this_ptr->hide_preedit_string ();
         }
@@ -687,8 +774,6 @@ M17NInstance::candidates_done_cb (MInputContext *ic, MSymbol command)
 static MSymbol
 __key_to_symbol (const KeyEvent &key)
 {
-    if (key.is_key_release ()) return Mnil;
-
     int mask = 0;
     String keysym;
 
@@ -727,7 +812,7 @@ __key_to_symbol (const KeyEvent &key)
     if (mask & SCIM_KEY_ControlMask)
         keysym = String ("C-") + keysym;
     if (mask & SCIM_KEY_AltMask)
-        keysym = String ("A-") + keysym;
+        keysym = String ("M-") + keysym;
 
     return msymbol (keysym.c_str ());
 }
